#ifndef DRIVER_INTERFACE_H
#define DRIVER_INTERFACE_H

#include <stdint.h>
#include <stdbool.h>

typedef enum DEVICE_CAPACITY
{
    DEVICE_CAPACITY_MOUSE,
    DEVICE_CAPACITY_WIRELESS,
} device_capacity_t;

void driver_init(); // Initialization
uint32_t driver_getkey(); // Calls to this function must return USB keys that this driver registers, until 0

device_capacity_t driver_get_capacity(void* handle); // Returns device capacity (see above)
char* driver_get_name(void* handle); // Returns peripheral name
char* driver_get_manufacturer(void* handle); // Returns peripheral manufacturer
char* driver_get_image(void* handle); // Returns peripheral image

/* Mouse drivers */
#define MAX_DPI_LEVELS 10
struct MOUSE_DPI_LEVELS
{
    unsigned int max_level_count;
    unsigned int level_count;
    unsigned int level_current;
    bool led_available;
    bool xy_available;
    struct MOUSE_DPI_LEVEL
    {
        unsigned int dpi_x;
        unsigned int dpi_y;
        unsigned char r;
        unsigned char g;
        unsigned char b;
    } level[MAX_DPI_LEVELS];
};
int driver_mouse_dpi_get(void* handle, struct MOUSE_DPI_LEVELS* output);

int driver_mouse_motion_sync_get(void* handle, bool* output);
int driver_mouse_angle_snap_get(void* handle, bool* output);

/* Wireless drivers */
int driver_wireless_battery_state_get(void* handle, int* battery_level, bool* charging);
int driver_wireless_connection_type_get(void* handle, bool* output);

#endif