#include "mouse-panel.h"

struct _MousePanel
{
    Panel parent_instance;

    GtkImage* mouse_image;
    GtkLabel* mouse_manufacturer;
    GtkLabel* mouse_name;
    AdwPreferencesGroup* dpi_preference_group;
    AdwButtonContent* dpi_stage_add_button;
    AdwSwitchRow* motion_sync_switchrow;
    AdwSwitchRow* angle_snap_switchrow;
};

G_DEFINE_TYPE (MousePanel, mouse_panel, panel_get_type())

static void
mouse_panel_class_init(MousePanelClass* klass)
{
	GtkWidgetClass* widget_class = GTK_WIDGET_CLASS(klass);

	gtk_widget_class_set_template_from_resource(widget_class, "/v/ginput/panel-mouse.ui");

    gtk_widget_class_bind_template_child(widget_class, MousePanel, mouse_image);
    gtk_widget_class_bind_template_child(widget_class, MousePanel, mouse_manufacturer);
    gtk_widget_class_bind_template_child(widget_class, MousePanel, mouse_name);
    gtk_widget_class_bind_template_child(widget_class, MousePanel, dpi_preference_group);
    gtk_widget_class_bind_template_child(widget_class, MousePanel, dpi_stage_add_button);
    gtk_widget_class_bind_template_child(widget_class, MousePanel, motion_sync_switchrow);
    gtk_widget_class_bind_template_child(widget_class, MousePanel, angle_snap_switchrow);
}

static void
mouse_panel_init(MousePanel* self)
{
	gtk_widget_init_template(GTK_WIDGET(self));
}

Panel*
mouse_panel_new(void)
{
    return g_object_new(mouse_panel_get_type(), NULL);
}

void mouse_panel_set_device(MousePanel* self, device_t* device)
{
    // Set device image
    char* image = device_get_image(device);
    gtk_image_set_from_file(self->mouse_image, image);
    free(image);

    // Set device manufacturer
    char* (*getmanufacturer_fn)(void*) = dlsym(device_driver(device), "driver_get_manufacturer");
    gtk_label_set_label(self->mouse_manufacturer, getmanufacturer_fn(device_handle(device)));

    // Set device name
    // TODO inner window title
    char* device_name = device_get_name(device);
    gtk_label_set_label(self->mouse_name, device_name);

    // Set mouse dpi
    struct MOUSE_DPI_LEVELS dpi;
    int (*driver_mouse_dpi_get)(void*, void*) = dlsym(device_driver(device), "driver_mouse_dpi_get");
    int dpi_res = driver_mouse_dpi_get(device_handle(device), &dpi);
    if(!dpi_res)
    {
        // Set dpi used/max label in add button
        char dpi_used_max[10];
        sprintf(dpi_used_max, "%u/%u", dpi.level_count, dpi.max_level_count);
        adw_button_content_set_label(self->dpi_stage_add_button, dpi_used_max);

        GtkCheckButton* group = 0;
        // Set dpi list
        for(size_t i = 0; i < dpi.level_count; i++)
        {
            AdwActionRow* row = ADW_ACTION_ROW(adw_action_row_new());
            
            // Add radiobutton
            if(dpi.level_count > 1)
            {
                GtkCheckButton* radio_button = GTK_CHECK_BUTTON(gtk_check_button_new());
                if(!group) group = radio_button;
                else gtk_check_button_set_group(radio_button, group);

                if(dpi.level_current == i)
                    gtk_check_button_set_active(radio_button, true);

                adw_action_row_add_prefix(row, GTK_WIDGET(radio_button));
            }

            // Convert dpi int to string, and set it as title
            char dpi_str[15];
            sprintf(dpi_str, "%u DPI", dpi.level[i].dpi_x);
            adw_preferences_row_set_title(ADW_PREFERENCES_ROW(row), dpi_str);

            //gtk_list_box_row_set_activatable(GTK_LIST_BOX_ROW(row), false);

            adw_preferences_group_add(self->dpi_preference_group, GTK_WIDGET(row));
        }
    }

    // Set mouse 'motion sync' feature
    bool motion_sync = false;
    int (*driver_mouse_motion_sync_get)(void*, bool*) = dlsym(device_driver(device), "driver_mouse_motion_sync_get");
    int motionsync_res = driver_mouse_motion_sync_get(device_handle(device), &motion_sync);
    if(!motionsync_res)
        adw_switch_row_set_active(self->motion_sync_switchrow, motion_sync);

    // Set mouse 'angle snap' feature
    bool angle_snap = false;
    int (*driver_mouse_angle_snap_get)(void*, bool*) = dlsym(device_driver(device), "driver_mouse_angle_snap_get");
    int anglesnap_res = driver_mouse_angle_snap_get(device_handle(device), &angle_snap);
    if(!anglesnap_res)
        adw_switch_row_set_active(self->angle_snap_switchrow, angle_snap);
}
